#include "XOPStandardHeaders.h"			// Include ANSI headers, Mac headers, IgorXOP.h, XOP.h and XOPSupport.h

// Author: Thomas Braun thomas dott braun ht virtuell - zuhause dott de
// License: LGPLv3 or later (proprietary licenses on request)

#include "Calculator.h"

namespace  {

  const int MIN_IGOR_VERSION            = 600;
  const int  FORCE_GLOBAL               = 1;
  const char GLOBAL_RESULT_STORE_NAME[] = "ans";
  const double INITIAL_VALUE            = DOUBLE_NAN;

  int RegisterCalc()
  {
    const char* cmdTemplate;
    const char* runtimeNumVarList;
    const char* runtimeStrVarList;

    // NOTE: If you change this template, you must change the CalcRuntimeParams structure as well.
    cmdTemplate = "Calc [number:expression]";
    runtimeNumVarList = "";
    runtimeStrVarList = "";
    return RegisterOperation(cmdTemplate, runtimeNumVarList, runtimeStrVarList, sizeof(CalcRuntimeParams), (void*)ExecuteCalc, 0);
  }

  // Register any operations with Igor
  int RegisterOperations()		
  {
    if (int result = RegisterCalc())
      return result;

    return 0;
  }

  /// Creates a global variable in targetFolder
  /// Objects of different type and located in targetFolder and named variableName will be killed 
  /// 
  /// @param targetFolder, folder in which the variable should be created
  /// @param variableName, name of the variable to create
  /// @param value, initial value of the variable
  /// @return zero on success an error code otherwise
  int CreateGlobalNumericVariable(DataFolderHandle targetFolder, const char* variableName, double value)
  {
    if (variableName == NULL || strlen(variableName) == 0)
      return OBJECT_DOES_NOT_EXIST;

    // check if it already exists
    int objType;
    DataObjectValue objValue;
    int ret = GetDataFolderObject(targetFolder,variableName,&objType,&objValue);
    if (ret == 0) // object exists
    {
      if (objType == VAR_OBJECT) // with the correct type so we just return
        return 0;

      // it exists and has the wrong type so we have to delete it
      ret = KillDataFolderObject(targetFolder,objType,variableName);
    }

    // the object does not exist

    // get current folder handle
    DataFolderHandle saveFolderHndl;
    ret = GetCurrentDataFolder(&saveFolderHndl);
    if(ret)
      return ret;

    // set current folder to targetFolder
    ret = SetCurrentDataFolder(targetFolder);
    if(ret)
      return ret;

    // create variable
    int retSetVar = SetIgorFloatingVar(variableName,&value,FORCE_GLOBAL);
    // don't handle errors from SetIgorFloatingVar now as we have to change the current datafolder back first

    // change current folder back
    ret = SetCurrentDataFolder(saveFolderHndl);

    return ( retSetVar != 0 ? retSetVar : ret );
  }

} // anonymous namespace 

extern "C" int ExecuteCalc(CalcRuntimeParamsPtr p)
{
  int ret = 0;

  // get rootfolder handle
  DataFolderHandle rootFolderHndl;
  ret = GetRootDataFolder(0, &rootFolderHndl);
  if(ret)
    return ret;

  // create global variable if necessary
  ret = CreateGlobalNumericVariable(rootFolderHndl,GLOBAL_RESULT_STORE_NAME,INITIAL_VALUE);
  if(ret)
    return ret;

  // update value in global variable
  if (p->expressionEncountered)
  { 
    DataObjectValue objValue;
    objValue.nv.realValue = p->expression;
    ret = SetDataFolderObject(rootFolderHndl,GLOBAL_RESULT_STORE_NAME,VAR_OBJECT,&objValue);
    if(ret)
      return ret;
  }

  // output value from global variable
  int objType;
  DataObjectValue objValue;
  ret = GetDataFolderObject(rootFolderHndl,GLOBAL_RESULT_STORE_NAME,&objType,&objValue);
  if(ret || objType != VAR_OBJECT)
    return ret;

  char buf[256];
  sprintf(buf,"%g" CR_STR,objValue.nv.realValue);
  XOPNotice(buf);

  return ret;
}

extern "C" void XOPEntry()
{	
	XOPIORecResult result = 0;
	
	//switch (GetXOPMessage())
 // {
	//	// We don't need to handle any messages for this XOP.
	//}
	SetXOPResult(result);
}

HOST_IMPORT int main(IORecHandle ioRecHandle)
{
	int result;
	
	XOPInit(ioRecHandle);
	SetXOPEntry(XOPEntry);
	
  // Requires Igor Pro 6.00 or later.
  if (igorVersion < MIN_IGOR_VERSION)
  {			
		SetXOPResult(OLD_IGOR);
		return EXIT_FAILURE;
	}

	if (result = RegisterOperations())
  {
		SetXOPResult(result);
		return EXIT_FAILURE;
	}
	
	SetXOPResult(0);
	return EXIT_SUCCESS;
}
